/*
 * Copyright (c) 2007 The Akuma Project
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * $Id: main.c 130 2007-05-29 12:54:04Z asmodai $
 */
/*
    		 Crown Copyright (c) 1997

    This TenDRA(r) Computer Program is subject to Copyright
    owned by the United Kingdom Secretary of State for Defence
    acting through the Defence Evaluation and Research Agency
    (DERA).  It is made available to Recipients with a
    royalty-free licence for its use, reproduction, transfer
    to other parties and amendment for any purpose not excluding
    product development provided that any such use et cetera
    shall be deemed to be acceptance of the following conditions:-

        (1) Its Recipients shall ensure that this Notice is
        reproduced upon any copies or amended versions of it;

        (2) Any amended version of it shall be clearly marked to
        show both the nature of and the organisation responsible
        for the relevant amendment or amendments;

        (3) Its onward transfer from a recipient to another
        party shall be deemed to be that party's acceptance of
        these conditions;

        (4) DERA gives no warranty or assurance as to its
        quality or suitability for any purpose and DERA accepts
        no liability whatsoever in relation to any use to which
        it may be put.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "char.h"
#include "error.h"
#include "lex.h"
#include "output.h"
#include "syntax.h"


/*
    MAIN ROUTINE

    This is the main routine.  It processes the command-line options,
    reads the input file, and writes the output files.
*/

int
main(int argc, char **argv)
{
	int a;
	int key = 0;
	int too_many = 0;
	char *input = NULL;
	char *output = NULL;

	/* Process arguments */
	set_progname(argv[0], "1.2");
	for (a = 1; a < argc; a++) {
		char *arg = argv[a];
		if (arg[0] == '-' && arg[1]) {
			int known = 0;
			switch (arg[1]) {
			case 'k': {
				if (arg[2]) {
					break;
				}
				key = 1;
				known = 1;
				break;
			}
			case 'l': {
				sid_prefix = arg + 2;
				known = 1;
				break;
			}
			case 'v': {
				if (arg[2]) {
					break;
				}
				report_version();
				known = 1;
				break;
			}
			}
			if (!known) {
				error(ERROR_WARNING, "Unknown option, '%s'",
				      arg);
			}
		} else {
			if (input == NULL) {
				input = arg;
			} else if (output == NULL) {
				output = arg;
			} else {
				too_many = 1;
			}
		}
	}

	/* Check arguments */
	if (input == NULL) {
		error(ERROR_FATAL, "Not enough arguments");
	}
	if (too_many) {
		error(ERROR_WARNING, "Too many arguments");
	}

	/* Process input file */
	process_file(input);

	/* Generate output */
	if (exit_status == EXIT_SUCCESS) {
		if (white_space == NULL) {
			white_space = make_string(" \t\n");
		}
		if (output == NULL || strcmp(output, "-") == 0) {
			lex_output = stdout;
			output = NULL;
		} else {
			lex_output = fopen(output, "w");
			if (lex_output == NULL) {
				error(ERROR_FATAL,
				      "Can't open output file, %s", output);
			}
		}
		if (key) {
			output_keyword();
		} else {
			output_all();
		}
		if (output) {
			fclose(lex_output);
		}
	} else {
		error(ERROR_FATAL, "Terminating due to previous errors");
	}
	return (exit_status);
}
